@@ -1,36 +1,34 @@
 #![allow(clippy::mutex_atomic)]
 
-use crate::basics::io_handler::EventSource;
-use crate::basics::Time;
+use crate::basics::EventSource;
+use crate::configuration::time::{init_start_time, TimeRepresentation};
 use crate::storage::Value;
+use crate::Time;
 use csv::{ByteRecord, Reader as CSVReader, Result as ReaderResult, StringRecord};
 use rtlola_frontend::mir::{RtLolaMir, Type};
 use std::error::Error;
 use std::fs::File;
 use std::io::stdin;
-use std::time::{Duration, Instant, SystemTime, UNIX_EPOCH};
+use std::path::PathBuf;
+use std::time::SystemTime;
 
+/// Configures the input source for the [CsvEventSource].
 #[derive(Debug, Clone)]
-enum TimeHandling {
-    RealTime { start: Instant },
-    FromFile { start: Option<SystemTime> },
-    Delayed { delay: Duration, time: Time },
+pub struct CsvInputSource {
+    /// The index of column in which the time information is given
+    /// If none the column named 'time' is chosen.
+    pub time_col: Option<usize>,
+    /// Specifies the input channel of the source.
+    pub kind: CsvInputSourceKind,
 }
 
+/// Sets the input channel of the [CsvEventSource]
 #[derive(Debug, Clone)]
-pub enum CsvInputSource {
+pub enum CsvInputSourceKind {
+    /// Use the std-in as an input channel
     StdIn,
-    File { path: String, delay: Option<Duration>, time_col: Option<usize> },
-}
-
-impl CsvInputSource {
-    pub fn file(path: String, delay: Option<Duration>, time_col: Option<usize>) -> CsvInputSource {
-        CsvInputSource::File { path, delay, time_col }
-    }
-
-    pub fn stdin() -> CsvInputSource {
-        CsvInputSource::StdIn
-    }
+    /// Use the specified file as an input channel
+    File(PathBuf),
 }
 
 #[derive(Debug, Clone)]
@@ -99,39 +97,34 @@ impl ReaderWrapper {
     }
 }
 
+///Parses events in CSV format.
 #[derive(Debug)]
-pub struct CsvEventSource {
+pub struct CsvEventSource<InputTime: TimeRepresentation> {
     reader: ReaderWrapper,
     record: ByteRecord,
     mapping: CsvColumnMapping,
     in_types: Vec<Type>,
-    timer: TimeHandling,
+    timer: InputTime,
 }
 
-impl CsvEventSource {
+impl<InputTime: TimeRepresentation> CsvEventSource<InputTime> {
     pub(crate) fn setup(
         src: &CsvInputSource,
+        timer: InputTime,
         ir: &RtLolaMir,
-        start_time: Instant,
-    ) -> Result<Box<dyn EventSource>, Box<dyn Error>> {
-        use CsvInputSource::*;
-        let (mut wrapper, time_col) = match src {
-            StdIn => (ReaderWrapper::Std(CSVReader::from_reader(stdin())), None),
-            File { path, time_col, .. } => (ReaderWrapper::File(CSVReader::from_path(path)?), *time_col),
+        start_time: Option<SystemTime>,
+    ) -> Result<Box<dyn EventSource<InputTime>>, Box<dyn Error>> {
+        let CsvInputSource { time_col, kind } = src;
+        let mut wrapper = match kind {
+            CsvInputSourceKind::StdIn => ReaderWrapper::Std(CSVReader::from_reader(stdin())),
+            CsvInputSourceKind::File(path) => ReaderWrapper::File(CSVReader::from_path(path)?),
         };
 
         let stream_names: Vec<&str> = ir.inputs.iter().map(|i| i.name.as_str()).collect();
-        let mapping = CsvColumnMapping::from_header(stream_names.as_slice(), wrapper.get_header()?, time_col);
+        let mapping = CsvColumnMapping::from_header(stream_names.as_slice(), wrapper.get_header()?, *time_col);
         let in_types: Vec<Type> = ir.inputs.iter().map(|i| i.ty.clone()).collect();
 
-        use TimeHandling::*;
-        let timer = match src {
-            StdIn => RealTime { start: start_time },
-            File { delay, .. } => match delay {
-                Some(d) => Delayed { delay: *d, time: Duration::default() },
-                None => FromFile { start: None },
-            },
-        };
+        init_start_time::<InputTime>(start_time);
 
         Ok(Box::new(CsvEventSource { reader: wrapper, record: ByteRecord::new(), mapping, in_types, timer }))
     }
@@ -174,24 +167,8 @@ impl CsvEventSource {
     }
 
     fn get_time(&mut self) -> Time {
-        use self::TimeHandling::*;
-        match self.timer {
-            RealTime { start } => Instant::now() - start,
-            FromFile { start } => {
-                let now = self.read_time().unwrap();
-                match start {
-                    None => {
-                        self.timer = FromFile { start: Some(now) };
-                        Time::default()
-                    }
-                    Some(start) => now.duration_since(start).expect("Time did not behave monotonically!"),
-                }
-            }
-            Delayed { delay, ref mut time } => {
-                *time += delay;
-                *time
-            }
-        }
+        let str = self.str_for_time().unwrap().to_string();
+        self.timer.parse(&str).unwrap()
     }
 
     fn read_event(&self) -> Vec<Value> {
@@ -222,7 +199,7 @@ impl CsvEventSource {
     }
 }
 
-impl EventSource for CsvEventSource {
+impl<InputTime: TimeRepresentation> EventSource<InputTime> for CsvEventSource<InputTime> {
     fn has_event(&mut self) -> bool {
         self.read_blocking().unwrap_or_else(|e| {
             eprintln!("error: failed to read data. {}", e);
@@ -235,40 +212,4 @@ impl EventSource for CsvEventSource {
         let time = self.get_time();
         (event, time)
     }
-
-    fn read_time(&self) -> Option<SystemTime> {
-        let time_str = self.str_for_time()?;
-        let mut time_str_split = time_str.split('.');
-        let secs_str: &str = match time_str_split.next() {
-            Some(s) => s,
-            None => {
-                eprintln!("error: problem with data source; failed to parse time string {}.", time_str);
-                std::process::exit(1)
-            }
-        };
-        let secs = match secs_str.parse::<u64>() {
-            Ok(u) => u,
-            Err(e) => {
-                eprintln!("error: problem with data source; failed to parse time string {}: {}", time_str, e);
-                std::process::exit(1)
-            }
-        };
-        let d: Duration = if let Some(nanos_str) = time_str_split.next() {
-            let mut chars = nanos_str.chars();
-            let mut nanos: u32 = 0;
-            for _ in 1..=9 {
-                nanos *= 10;
-                if let Some(c) = chars.next() {
-                    if let Some(d) = c.to_digit(10) {
-                        nanos += d;
-                    }
-                }
-            }
-            assert!(time_str_split.next().is_none());
-            Duration::new(secs, nanos)
-        } else {
-            Duration::from_nanos(secs)
-        };
-        Some(UNIX_EPOCH + d)
-    }
 }
