@@ -1,8 +1,9 @@
 #![allow(clippy::mutex_atomic)]
 
 use crate::basics::io_handler::EventSource;
-use crate::basics::Time;
+use crate::configuration::time::{init_start_time, TimeRepresentation};
 use crate::storage::Value;
+use crate::Time;
 use etherparse::{
     Ethernet2Header, InternetSlice, Ipv4Header, Ipv6Header, LinkSlice, SlicedPacket, TcpHeader, TransportSlice,
     UdpHeader,
@@ -10,10 +11,12 @@ use etherparse::{
 use ip_network::IpNetwork;
 use pcap::{Activated, Capture, Device, Error as PCAPError};
 use rtlola_frontend::mir::RtLolaMir;
+use std::convert::TryFrom;
 use std::error::Error;
 use std::net::IpAddr;
+use std::path::PathBuf;
 use std::str::FromStr;
-use std::time::{Duration, Instant, SystemTime, UNIX_EPOCH};
+use std::time::SystemTime;
 
 // ################################
 // Packet parsing functions
@@ -416,34 +419,42 @@ fn get_packet_protocol(packet: &SlicedPacket) -> Value {
 // ################################
 // Event Source Handling
 // ################################
+/// Determines the input source for network packets.
 #[derive(Debug, Clone)]
 pub enum PCAPInputSource {
-    Device { name: String, local_network: String },
-    File { path: String, delay: Option<Duration>, local_network: String },
-}
-
-enum TimeHandling {
-    RealTime { start: Instant },
-    FromFile { start: Option<SystemTime> },
-    Delayed { delay: Duration, time: Time },
+    /// Use the specified device for packet input.
+    Device {
+        /// The name of the device to use.
+        name: String,
+        /// The description of your local network IP address range in CIDR-Notation.
+        local_network: String,
+    },
+    /// Use the given file for packet input.
+    File {
+        /// The path to the PCAP file.
+        path: PathBuf,
+        /// The description of your local network IP address range in CIDR-Notation.
+        local_network: String,
+    },
 }
 
+/// Parses events from network packets.
 #[allow(missing_debug_implementations)] // Capture -> PcapOnDemand does not implement Debug.
-pub struct PCAPEventSource {
+pub struct PCAPEventSource<InputTime: TimeRepresentation> {
     capture_handle: Capture<dyn Activated>,
-    timer: TimeHandling,
+    timer: InputTime,
     mapping: Vec<Box<dyn Fn(&SlicedPacket) -> Value>>,
 
     event: Option<(Vec<Value>, Time)>,
-    last_timestamp: Option<SystemTime>,
 }
 
-impl PCAPEventSource {
+impl<InputTime: TimeRepresentation> PCAPEventSource<InputTime> {
     pub(crate) fn setup(
         src: &PCAPInputSource,
+        timer: InputTime,
         ir: &RtLolaMir,
-        start_time: Instant,
-    ) -> Result<Box<dyn EventSource>, Box<dyn Error>> {
+        start_time: Option<SystemTime>,
+    ) -> Result<Box<dyn EventSource<InputTime>>, Box<dyn Error>> {
         let capture_handle = match src {
             PCAPInputSource::Device { name, .. } => {
                 let all_devices = Device::list()?;
@@ -485,14 +496,7 @@ impl PCAPEventSource {
             }
         };
 
-        use TimeHandling::*;
-        let timer = match src {
-            PCAPInputSource::Device { .. } => RealTime { start: start_time },
-            PCAPInputSource::File { delay, .. } => match delay {
-                Some(d) => Delayed { delay: *d, time: Duration::default() },
-                None => FromFile { start: None },
-            },
-        };
+        init_start_time::<InputTime>(start_time);
         let input_names: Vec<String> = ir.inputs.iter().map(|i| i.name.clone()).collect();
 
         // Generate Mapping that given a parsed packet returns the value for the corresponding input stream
@@ -644,7 +648,7 @@ impl PCAPEventSource {
             mapping.push(val);
         }
 
-        Ok(Box::new(PCAPEventSource { capture_handle, timer, mapping, event: None, last_timestamp: None }))
+        Ok(Box::new(PCAPEventSource { capture_handle, timer, mapping, event: None }))
     }
 
     fn process_packet(&mut self) -> Result<bool, Box<dyn Error>> {
@@ -656,12 +660,12 @@ impl PCAPEventSource {
             },
         };
 
-        use std::convert::TryInto;
-        let d = Duration::new(
-            raw_packet.header.ts.tv_sec.try_into().unwrap(),
-            (raw_packet.header.ts.tv_usec * 1000).try_into().unwrap(),
+        let (secs, nanos) = (
+            u64::try_from(raw_packet.header.ts.tv_sec).unwrap(),
+            u32::try_from(raw_packet.header.ts.tv_usec * 1000).unwrap(),
         );
-        self.last_timestamp = Some(UNIX_EPOCH + d);
+        let time_str = format!("{}.{:09}", secs, nanos);
+        let time = self.timer.parse(&time_str)?;
 
         let p = SlicedPacket::from_ethernet(raw_packet.data);
         //Todo (Florian): Track underlying error
@@ -675,33 +679,13 @@ impl PCAPEventSource {
             event.push(parse_function(&packet));
         }
 
-        //compute duration since start
-        use TimeHandling::*;
-        let dur: Time = match self.timer {
-            RealTime { start } => Instant::now() - start,
-            FromFile { start } => {
-                let now = self.last_timestamp.unwrap();
-                match start {
-                    None => {
-                        self.timer = FromFile { start: Some(now) };
-                        Time::default()
-                    }
-                    Some(start) => now.duration_since(start).expect("Time did not behave monotonically!"),
-                }
-            }
-            Delayed { delay, ref mut time } => {
-                *time += delay;
-                *time
-            }
-        };
-
-        self.event = Some((event, dur));
+        self.event = Some((event, time));
 
         Ok(true)
     }
 }
 
-impl EventSource for PCAPEventSource {
+impl<InputTime: TimeRepresentation> EventSource<InputTime> for PCAPEventSource<InputTime> {
     fn has_event(&mut self) -> bool {
         self.process_packet().unwrap_or_else(|e| {
             eprintln!("error: failed to process packet. {}", e);
@@ -717,8 +701,4 @@ impl EventSource for PCAPEventSource {
             std::process::exit(1);
         }
     }
-
-    fn read_time(&self) -> Option<SystemTime> {
-        self.last_timestamp
-    }
 }
